/**
 * SPDX-FileCopyrightText: 2024 Sascha Nitsch (grumpydeveloper) https://contentnation.net/@grumpydevelop
 * SPDX-License-Identifier: GPL-3.0-or-later
 * @author Author: Sascha Nitsch (grumpydeveloper)
 **/

import { PresentationMaker } from '../presentationmaker';
import { RTModule } from './rtmodule';
import { RTNode } from './rtnode';
import { RTImg } from './rtimg';
import { RTCalc } from './rtcalc';
import { RTText } from './rttext';
import { RTOptions } from './rtoptions';
import { RTPage } from './rtpage';

/**
 * the root of render tree
 * @internal
 */
export class RenderTree {
  /** handle to main instance */
  private main: PresentationMaker;
  /** our child elements */
  private elements: RTNode[];
  /** definitions from input */
  private definitions: Map<string, string>;

  /**
   * constructor
   * @param pm handle to main instance
   * @param other optional other tree to copy data from
   */
  constructor(pm: PresentationMaker, other: RenderTree = null) {
    this.main = pm;
    this.elements = [];
    this.definitions = new Map<string, string>();
    if (other !== null) {
      other.elements.forEach((e) => {
        this.elements.push(e.clone());
      });
    }
  }

  /**
   * create elements from given parent
   * @param baseElem root element to create new render tree nodes
   */
  createFromElem(baseElem: HTMLElement) {
    const nodes = baseElem.childNodes;
    nodes.forEach((node) => {
      if (node.nodeType === 1) {
        const newNode = this.createNodeFromElem(<HTMLElement>node);
        if (newNode !== null) {
          this.elements.push(newNode);
        }
      }
    });
  }


  /**
   * create a new node from given element
   * 
   * currently supports 'calc', 'img', 'options', 'page', 'text'
   * @param elem to create from
   */
  createNodeFromElem(elem: HTMLElement): RTNode {
    const name = elem.localName;
    // final types
    switch (name) {
      case 'calc':
        return RTCalc.fromElem(this, elem);
      case 'img':
        return RTImg.fromElem(this, elem);
      case 'options':
        return RTOptions.fromElem(this.main, this, elem);
      case 'page':
        return RTPage.fromElem(this.main, this, elem);
      case 'text':
        return RTText.fromElem(this, elem);
    }
    // still here, probably a module
    const tree = this.main.getNewModule(name);
    if (tree === null) {
      console.log('unknown module', name);
      return null;
    }
    tree.definitions.clear();
    // add attributes to node
    for (let i = 0; i < elem.attributes.length; ++i) {
      const attr = elem.attributes[i];
      tree.definitions.set(attr.localName, attr.textContent);
    }
    elem.childNodes.forEach((child) => {
      if (child.nodeType === 1) {
        const childElem = <HTMLElement>child;
        tree.definitions.set(childElem.localName, childElem.innerHTML);
      }
    });
    return new RTModule(this, tree);
  }

  /**
   * get current definition for given name
   * @param name name to look up
   * @returns value or ''
   */
  getDefinition(name: string): string {
    return this.main.getDefinition(name);
  }

  /**
   * get node with given index
   * @param num index
   * @return node or null if invalid index
   */
  getElement(num: number): RTNode {
    if (num >= 0 && num <= this.elements.length) {
      return this.elements[num];
    }
    return null;
  }

  /**
   * return the number of elements in the root tree
   */
  getElementCount(): number {
    return this.elements.length;
  }

  /**
   * get main instance
   * @returns main instance
   */
  getMain(): PresentationMaker {
    return this.main;
  }

  /**
   * pop defintion list to remove last definition from stack
   */
  pop() {
    this.main.pop();
  }

  /**
   * push a new empty definition list to stack
   */
  push(): void {
    this.main.push();
  }

  /**
   * render tree to given target
   * @param target target to render into
   */
  render(target: HTMLElement) {
    this.main.push();
    // save current definitions on the stack
    this.definitions.forEach((v, k) => {
      this.main.set(k, v);
    });
    this.elements.forEach((element) => {
      element.render(target);
    });
    this.main.pop();
  }

  /**
   * set definition
   * @param key key name
   * @param value value data
   */
  set(key: string, value: string) {
    this.main.set(key, value);
  }

};
