/**
 * SPDX-FileCopyrightText: 2024 Sascha Nitsch (grumpydeveloper) https://contentnation.net/@grumpydevelop
 * SPDX-License-Identifier: GPL-3.0-or-later
 * @author Author: Sascha Nitsch (grumpydeveloper)
 * @internal
 **/

import { RTNode } from './rtnode';
import { RenderTree } from './rendertree';
import { PresentationMaker } from '../presentationmaker';

/**
 * view transformations as strings
 */
export interface TransformationS {
  /** position on HTML space */
  position: {x: string, y: string, z: string};
  /** rotation in HTML space */
  rotation: {x: string, y: string, z: string};
  /** page size in pixels */
  size: {w: string, h: string};
  /** view position in HTML space */
  viewpos: {x: string, y: string, z: string};
  /** view size in pixels */
  viewSize: {w: string, h: string};
  /** view rotation in degree */
  viewRotation: {x: string, y: string, z: string};
}

/**
 * view transformation as numbers
 */
export interface RTTransformation {
  /** view position */
  position: {x: number, y: number, z: number};
  /** view size */
  size: {w: number, h: number};
  /** view rotation in degrees */
  viewRotation: {x: number, y: number, z: number};
}

/**
 * page node
 */
export class RTPage extends RTNode {
  /** page id */
  private id: string;
  /** page name */
  private name: string;
  /** tree to render on page */
  private subtree: RenderTree;
  /** tranform informations */
  private transformation: TransformationS;

  /**
   * clone function
   * @return a copy of us
   */
  clone(): RTPage {
    return new RTPage(
      this.root,
      this.subtree,
      this.id,
      this.transformation.size.w,
      this.transformation.size.h,
      this.transformation.position.x,
      this.transformation.position.y,
      this.transformation.position.z,
      this.transformation.rotation.y,
      this.transformation.rotation.y,
      this.transformation.rotation.z,
      this.transformation.viewpos.x,
      this.transformation.viewpos.y,
      this.transformation.viewpos.z,
      this.transformation.viewSize.w,
      this.transformation.viewSize.h,
      this.transformation.viewRotation.x,
      this.transformation.viewRotation.y,
      this.transformation.viewRotation.z,
      this.name
    )
  }

  /**
   * constructor
   * @param root parent RenderTree
   * @param subtree RenderTree for page content
   * @param id page id
   * @param width page width
   * @param height page height
   * @param x x position
   * @param y y position
   * @param z z position
   * @param rx rotation on X-axis
   * @param ry rotation on Y-axis
   * @param rz rotation on Z-axis
   * @param vx view position on X-axis
   * @param vy view position on Y-axis
   * @param vz view position on Z-axis
   * @param vrx view rotation on X-axis
   * @param vry view rotation on Y-axis
   * @param vrz view rotation on Z-axis
   */
  constructor(root: RenderTree, subtree: RenderTree, id: string, width: string, height: string, x: string, y: string, z: string, rx: string, ry: string, rz:string, vx: string, vy: string, vz: string, vw: string, vh: string, vrx: string, vry: string, vrz: string, name: string) {
    super(root);
    this.subtree = subtree;
    this.id = id;
    this.transformation = {
      size: { w: width, h: height},
      position: {x: x, y: y, z: z},
      rotation: {x: rx, y: ry, z: rz},
      viewpos: {x: vx, y: vy, z: vz},
      viewSize: {w: vw, h: vh},
      viewRotation: {x: vrx, y: vry, z: vrz}
    };
    this.name = name;
  }

  /**
   * create RTPage from given node into given RenderTree
   * @param root render tree to insert to
   * @param node input node
   * @returns new RTOptions node
   */
  static fromElem(main:PresentationMaker, root: RenderTree, node: HTMLElement): RTPage {
    const subtree = new RenderTree(main);
    subtree.createFromElem(node);
    return new RTPage(
      root,
      subtree,
      node.getAttribute('id') || '',
      node.getAttribute('width') || '${width}',
      node.getAttribute('height') || '${height}',
      node.getAttribute('x') || '0',
      node.getAttribute('y') || '0',
      node.getAttribute('z') || '0',
      node.getAttribute('rx') || '0',
      node.getAttribute('ry') || '0',
      node.getAttribute('rz') || '0',
      node.getAttribute('vx') || '',
      node.getAttribute('vy') || '',
      node.getAttribute('vz') || '',
      node.getAttribute('vwidth') || '',
      node.getAttribute('vheight') || '',
      node.getAttribute('vrx') || '',
      node.getAttribute('vry') || '',
      node.getAttribute('vrz') || '',
      node.getAttribute('name') || '${page}',
    );
  }

  /**
   * get page name
   * @returns page name
   */
  getName(): string {
    return this.name;
  }
  
  /**
   * get view transformation
   * @return RTTransformation
   */
  getView(): RTTransformation {
    let position;
    if (this.transformation.viewpos.x !== '' || this.transformation.viewpos.y !== '' || this.transformation.viewpos.z !=='') {
      position = {
        x: this.calc(this.resolveString(this.transformation.viewpos.x)),
        y: this.calc(this.resolveString(this.transformation.viewpos.y)),
        z: this.calc(this.resolveString(this.transformation.viewpos.z))
      };
    } else {
      position = {
        x: this.calc(this.resolveString(this.transformation.position.x)),
        y: this.calc(this.resolveString(this.transformation.position.y)),
        z: this.calc(this.resolveString(this.transformation.position.z))
      }
    }
    let size;
    if (this.transformation.viewSize.w !== '' || this.transformation.viewSize.h !== '') {
      size = {
        w: this.calc(this.resolveString(this.transformation.viewSize.w)),
        h: this.calc(this.resolveString(this.transformation.viewSize.h))
      };
    } else {
      size = {
        w: this.calc(this.resolveString(this.transformation.size.w)),
        h: this.calc(this.resolveString(this.transformation.size.h))
      }
    }
    let viewRotation;
    if (this.transformation.viewRotation.x !== '' || this.transformation.viewRotation.y !== '' || this.transformation.viewRotation.z !== '') {
      viewRotation = {
        x: this.calc(this.resolveString(this.transformation.viewRotation.x)),
        y: this.calc(this.resolveString(this.transformation.viewRotation.y)),
        z: this.calc(this.resolveString(this.transformation.viewRotation.z))
      }
    } else {
      viewRotation = {
        x: 0,
        y: 0,
        z: 0
      }
    }
    return {
      position: position,
      size: size,
      viewRotation: viewRotation
    };
  }

  /**
   * render our element into target
   * @param target target to render into
   */
  render(target: HTMLElement): void {
    this.root.set('page', (parseInt(this.root.getDefinition('page')) + 1).toString());
    const pageDiv = document.createElement('article');
    pageDiv.classList.add('page');
    pageDiv.classList.add('inactive');
    let transform = '';
    const x = this.calc(this.resolveString(this.transformation.position.x));
    const y = this.calc(this.resolveString(this.transformation.position.y));
    const z = this.calc(this.resolveString(this.transformation.position.z));
    if (x || y || z) {
      transform += 'translate3d(' + x + 'px, ' + y + 'px, ' + z + 'px)';
    }
    const rx = -this.calc(this.resolveString(this.transformation.rotation.x));
    const ry = -this.calc(this.resolveString(this.transformation.rotation.y));
    const rz = -this.calc(this.resolveString(this.transformation.rotation.z));
    if (rx) {
      transform += 'rotateX(' + rx + 'deg)';
    }
    if (ry) {
      transform += 'rotateY(' + ry + 'deg)';
    }
    if (rz) {
      transform += 'rotateZ(' + rz + 'deg)';
    }
    if (transform !== '') {
      pageDiv.style.transform = transform;
    }
    const width = this.calc(this.resolveString(this.transformation.size.w)).toString();
    const height = this.calc(this.resolveString(this.transformation.size.h)).toString();
    pageDiv.style.width = width + 'px';
    pageDiv.style.height = height + 'px';
    if (this.id) {
      pageDiv.id = this.id;
    }
    target.append(pageDiv);
    this.root.push()
    this.subtree.set('width', width);
    this.subtree.set('height', height);
    this.subtree.render(pageDiv);
    this.root.pop()
  }
}
